/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.generator.common.utils.SourceModelElementFactory.createByteContentUnitForPluginFile;

import org.fortiss.af3.generator.common.model.c.CSourcePackage;
import org.fortiss.af3.generator.common.model.source.ByteContentUnit;
import org.fortiss.af3.platform.raspberry.AF3PlatformRaspberryActivator;

/**
 * Class for copying the header files to the target folder.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
final class HeaderCopyGenerator {
	/** Adds the header files to the given target {@link CSourcePackage}. */
	public static void copyHeaderFiles(CSourcePackage incLibPack) throws Exception {
		incLibPack.addUnit(copyAF3Hal("af3.h"));
		incLibPack.addUnit(copyAF3Hal("cancatalog.h"));
		incLibPack.addUnit(copyAF3Hal("caninbox.h"));
		incLibPack.addUnit(copyAF3Hal("canoutbox.h"));
		incLibPack.addUnit(copyAF3Hal("cansocket.h"));
		incLibPack.addUnit(copyAF3Hal("canthread.h"));
		incLibPack.addUnit(copyAF3Hal("debugprint.h"));
		incLibPack.addUnit(copyAF3Hal("gamepad.h"));
		incLibPack.addUnit(copyAF3Hal("protocol_can.h"));
		incLibPack.addUnit(copyAF3Hal("protocol_control_center.h"));
		incLibPack.addUnit(copyAF3Hal("protocol_coordinator.h"));
		incLibPack.addUnit(copyAF3Hal("protocol_factory.h"));
		incLibPack.addUnit(copyAF3Hal("protocol_worker.h"));
		incLibPack.addUnit(copyAF3Hal("timeutil.h"));
	}

	/** Copies the AF3 HAL header file. */
	private static ByteContentUnit copyAF3Hal(String header) throws Exception {
		return copyHeader("af3pihal/", header);
	}

	/** Copies the brick header file. */
	private static ByteContentUnit copyBrick(String header) throws Exception {
		return copyHeader("brick/", header);
	}

	/** Copies the header file from the plugin path. */
	private static ByteContentUnit copyHeader(String subDir, String headerName) throws Exception {
		return createByteContentUnitForPluginFile(AF3PlatformRaspberryActivator.PLUGIN_ID,
				"code-gen-hal/inc/" + subDir, headerName, false);
	}
}
