/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.platform.raspberry.generator.templates.RasPiCTemplates.getMainCFile;

import java.util.List;

import org.conqat.lib.commons.collections.Pair;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.component.model.Port;
import org.fortiss.af3.generator.common.model.source.AbstractUnit;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.PlatformConnectorUnit;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;

/**
 * Separate class for generating the main.c file.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
class MainGenerator {
	/** Creates the main.c file. */
	public AbstractUnit createMain(RaspberryPi executionUnit,
			List<Pair<ExecutionUnit, Component>> deployedComponents,
			List<Pair<PlatformConnectorUnit, Port>> deployedPorts, ITransformationContext context) {
		String includes = createIncludes(deployedComponents);
		// TODO: syncbox code
		String initCode = createInitCode(deployedComponents);
		String workerCode = createWorkerCode(deployedComponents);
		return getMainCFile(executionUnit.getName(), executionUnit.isCoordinatorUnit(),
				executionUnit.getCanCoordinationID(), executionUnit.getCycleTime(), 250, includes,
				"// TODO\n", initCode, workerCode);
	}

	/** Creates the includes of the system headers. */
	private String createIncludes(List<Pair<ExecutionUnit, Component>> deployedComponents) {
		StringBuilder sb = new StringBuilder();
		for(Pair<ExecutionUnit, Component> p : deployedComponents) {
			Component c = p.getSecond();
			sb.append("#include <" + c.getName() + "_ID_" + c.getId() + ".h>\n");
		}
		return sb.toString();
	}

	/** Create the worker code. */
	private String createWorkerCode(List<Pair<ExecutionUnit, Component>> deployedComponents) {
		StringBuilder sb = new StringBuilder();
		for(Pair<ExecutionUnit, Component> p : deployedComponents) {
			Component c = p.getSecond();
			sb.append(makeCall("perform_step", c));
		}
		return sb.toString();
	}

	/** Create the initialize code. */
	private String createInitCode(List<Pair<ExecutionUnit, Component>> deployedComponents) {
		StringBuilder sb = new StringBuilder();
		for(Pair<ExecutionUnit, Component> p : deployedComponents) {
			Component c = p.getSecond();
			sb.append(makeCall("init", c));
		}
		return sb.toString();
	}

	/** Creates the function call code. */
	private String makeCall(String function, Component c) {
		StringBuilder sb = new StringBuilder();
		sb.append(function).append('_').append(c.getName());
		sb.append("_ID_").append(c.getId()).append("();\n");
		return sb.toString();
	}
}
